/****************************************************/
/*	H8/3048F-ONE MONITOR Program  					*/
/*		H8_3048F_MONITOR.c							*/
/*      Date	2009/04/01                        	*/
/*           	microkit I.Kurashima 				*/
/*                                                 	*/
/*      Rev   	Date           Changes          	*/
/*      1.00    2009/04/01     Original Version		*/
/****************************************************/

#include <stdio.h>
#include <sysio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "3048FONE.H"
#include "H8_MON_DEF.h"


/***********************************************************************
	C
************************************************************************/

void main(void){
	CpuInit();
	
	debug_mode();
}


/*----------------------------------------------------------------------
	CPȔ
-----------------------------------------------------------------------*/

void CpuInit(void){

//	**** bsc set ****

	BSC.ABWCR.BYTE = 0xF3;				// CS2,3 = 16bit bus

//	**** port set ****

	P1.DR.BYTE = 0x00;					// port1	Address A00--A07
	P1.DDR = 0xFF;
	
	P2.DR.BYTE = 0x00;					// port2	Address A08--A15
	P2.DDR = 0xFF;
	
//	P3.DR.BYTE = 0x00;					// port3	Data D15--D08
//	P3.DDR = 0x70;
	
//	P4.DR.BYTE = 0x00;					// port4	Data D07--D00
//	P4.DDR = 0xFF;
	
	P5.DR.BYTE = 0x00;					// port5	Address A16--A19
	P5.DDR = 0xFF;
	
	P6.DR.BYTE = 0x06;					// port6	nLWR, nHWR, nRD, nAS
	P6.DDR = 0xFE;						// 			BIT0 input
	
	P8.DR.BYTE = 0x00;					// port8	CS1, CS2, CS3 enable
	P8.DDR = 0xEE;						// 			BIT0 input
	
	P9.DR.BYTE = 0x00;					// port9    BIT5,4 output
	P9.DDR = 0xF0;

	PA.DR.BYTE = 0x00;					// portA	all input
	PA.DDR = 0x00;
	
	PB.DR.BYTE = 0x00;					// portB	all output
	PB.DDR = 0xFF;
}


/*----------------------------------------------------------------------
	debug mode
-----------------------------------------------------------------------*/

void debug_mode(void) {

	char	cend_flag;
	char	c, command[62], buf[16];
	unsigned char i, *p, *q;

	putchar(LF);
	puts("\033[34m");
	puts(" ******************************************");
	puts(" *  H8 3048F hardware debug monitor       *");
	puts(" *     microKit         Ikuo Kurashima    *");
	puts(" ******************************************");
	puts("\033[30m");

	while(1){
		cend_flag = 0;
		i = 0;
		
		sputs("\n> ");

		while (cend_flag == 0) {
			c = toupper(getchar());
		
			if((c == LF) || (c == CR) || (i >= 60)){
				command[i] = 0;
				cend_flag = 1;
				
				if(i >=60){
					puts("\033[31m");
					puts("**** too long COMMAND ****");
					puts("\033[30m");
					putchar(LF);
				}
			}
			else if(c == BS){
				if(i>0){
					i--;
					i--;
					putchar(' ');
					putchar(BS);
				}
			}
			else{
				command[i] = c;
			}
			i++;
		}

		p = strchr(command, ' ');						// R}h؂o
		
		if(p == NULL){
			p = strchr(command, '\0');
		}
		
		for(q = command, i = 0; p != q; q++, i++){
			buf[i] = *q;
		}
		
		buf[i] = '\0';
		
		while(*p == ' '){								// p[^擪o
			p++;
		}
	
		if(strcmp(buf, "DB") == 0){						// disp regster BYTE
			mon_DB(p);
		}
		else if(strcmp(buf, "DW") == 0){				// disp regster WORD
			mon_DW(p);
		}
		else if(strcmp(buf, "SB") == 0){				// set regster BYTE
			mon_SB(p);
		}
		else if(strcmp(buf, "SW") == 0){				// set regster WORD
			mon_SW(p);
		}

		else if(strcmp(buf, "DPB") == 0){				// disp port nBYTE
			mon_DPB(p);
		}
		else if(strcmp(buf, "DPW") == 0){				// disp port nWORD
			mon_DPW(p);
		}
		else if(strcmp(buf, "SPB") == 0){				// set port nBYTE
			mon_SPB(p);
		}
		else if(strcmp(buf, "SPW") == 0){				// set port nWORD
			mon_SPW(p);
		}

		else if(strcmp(buf, "D2B") == 0){				// disp port binary BYTE
			mon_D2B(p);
		}
		else if(strcmp(buf, "D2W") == 0){				// disp port binary WORD
			mon_D2W(p);
		}

		else if(strcmp(buf, "ANDB") == 0){				// and regster BYTE
			mon_ANDB(p);
		}
		else if(strcmp(buf, "ANDW") == 0){				// and regster WORD
			mon_ANDW(p);
		}
		else if(strcmp(buf, "ORB") == 0){				// or regster BYTE
			mon_ORB(p);
		}
		else if(strcmp(buf, "ORW") == 0){				// or regster WORD
			mon_ORW(p);
		}

		else if(strcmp(buf, "MASKB") == 0){				// mask BYTE
			mon_MASKB(p);
		}
		else if(strcmp(buf, "MASKW") == 0){				// MASK WORD
			mon_MASKW(p);
		}

		else if(strcmp(buf, "CHKB") == 0){				// check BYTE pattern
			mon_CHKB(p);
		}
		else if(strcmp(buf, "CHKW") == 0){				// check WORD pattern
			mon_CHKW(p);
		}


		else if(strcmp(buf, "CSET") == 0){				// disp color set
			mon_CSET(p);
		}
		else if(strcmp(buf, "//") == 0){				// comment
			;
		}
		else if(strcmp(buf, "?") == 0){					// disp help
			mon_help();
		}
		else if(command[0] |= 0){						// sR}h
			putchar(LF);
			puts("\033[31m");
			puts("** illegal command **");
			puts("\033[30m");
		}
	}
}


