/****************************************************/
/*	H8/3048F-ONE MONITOR Program  					*/
/*		H8_MON_SUB.c								*/
/*      Date	2009/04/01                        	*/
/*           	microkit I.Kurashima 				*/
/*                                                 	*/
/*      Rev   	Date           Changes          	*/
/*      1.00    2009/04/01     Original Version		*/
/****************************************************/


#include <stdio.h>
#include <sysio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include "H8_MON_DEF.h"


/********    mon_DB    ********/

void mon_DB(char *nParaStr){

	unsigned char	*nAddr;
	char			*nStr;
	unsigned int	i, nCnt;

	nAddr = (unsigned char *)strtoul(nParaStr, &nStr, 16);
	nStr += 1;
	
	nCnt = (unsigned int)strtoul(nStr, NULL, 10);
	
	putchar(LF);
	puts(" offset   0  1  2  3  4  5  6  7    8  9  A  B  C  D  E  F");
	puts(" ------+-------------------------+------------------------");

	putchar(' ');
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(": ");
	
	for(i=0; i<nCnt; i++){
		hextochar_disp(2, (unsigned long)(*nAddr));
		putchar(' ');
		nAddr++;

		if((i%16 == 15) && (i<(nCnt-1))){
			putchar(LF);
			putchar(' ');
			hextochar_disp(6, (unsigned long)nAddr);
			sputs(": ");
		}

		if((i%16 == 7) && (i<(nCnt-1))){
			sputs("- ");
		}
	}
	putchar(LF);
}	


/********    mon_DW    ********/

void mon_DW(char *nParaStr){

	unsigned short	*nAddr;
	char			*nStr;
	unsigned int	i, nCnt;

	nAddr = (unsigned short *)strtoul(nParaStr, &nStr, 16);
	nStr += 1;
	
	nCnt = (unsigned int)strtoul(nStr, NULL, 10);
	
	putchar(LF);
	puts(" offset     0    2    4    6      8    A    C    E");
	puts(" ------+---------------------+--------------------");

	putchar(' ');
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(": ");
	
	for(i=0; i<nCnt; i++){
		hextochar_disp(4, (unsigned long)(*nAddr));
		putchar(' ');
		nAddr++;

		if((i%8 == 7) && (i<(nCnt-1))){
			putchar(LF);
			putchar(' ');
			hextochar_disp(6, (unsigned long)nAddr);
			sputs(": ");
		}

		if((i%8 == 3) && (i<(nCnt-1))){
			sputs("- ");
		}
	}
	putchar(LF);
}	


/********    mon_SB    ********/

void mon_SB(char *nParaStr){

	unsigned char	*nAddr;
	char			*nDataStr;
	unsigned char	nData;

	nAddr = (unsigned char *)strtoul(nParaStr, &nDataStr, 16);
	nDataStr += 1;
	
	nData = (unsigned char)strtoul(nDataStr, NULL, 16);
	*nAddr = nData;

	putchar(LF);
	sputs(" Data set 0x");
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(" <-- 0x");
	hextochar_disp(2, (unsigned long)(nData));
	putchar(LF);
}


/********    mon_SW    ********/

void mon_SW(char *nParaStr){

	unsigned short	*nAddr;
	char			*nDataStr;
	unsigned short	nData;

	nAddr = (unsigned short *)strtoul(nParaStr, &nDataStr, 16);
	nDataStr += 1;
	
	nData = (unsigned short)strtoul(nDataStr, NULL, 16);
	*nAddr = nData;

	putchar(LF);
	sputs(" Data set 0x");
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(" <-- 0x");
	hextochar_disp(4, (unsigned long)(nData));
	putchar(LF);
}


/********    mon_DPB    ********/

void mon_DPB( char *nParaStr )
{
	unsigned char *nAddr;
	char *nCntStr;
	unsigned int nCnt;
	int i;

	nAddr = (unsigned char *)strtoul( nParaStr, &nCntStr, 16 );
	nCntStr += 1;
	nCnt = (unsigned int)strtoul( nCntStr, NULL, 10 );

	putchar(LF);
	putchar(' ');
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(": ");
	dectochar_disp(5, (unsigned long)nCnt);
	putchar(LF);
	puts(" -----------------------");
	putchar(' ');
	
	for(i=0; i<nCnt; i++){
		hextochar_disp(2, (unsigned long)(*nAddr));
		putchar(' ');

		if((i%8 == 7) && (i<(nCnt-1))){
			putchar(LF);
			putchar(' ');
		}
	}
	putchar(LF);
}	


/********    mon_DPW    ********/

void mon_DPW( char *nParaStr )
{
	unsigned short *nAddr;
	char *nCntStr;
	unsigned int nCnt;
	int i;

	nAddr = (unsigned short *)strtoul( nParaStr, &nCntStr, 16 );
	nCntStr += 1;
	nCnt = (unsigned int)strtoul( nCntStr, NULL, 10 );

	putchar(LF);
	putchar(' ');
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(": ");
	dectochar_disp(5, (unsigned long)nCnt);
	putchar(LF);
	puts(" ---------------------------------------");
	putchar(' ');
	
	for(i=0; i<nCnt; i++){
		hextochar_disp(4, (unsigned long)(*nAddr));
		putchar(' ');

		if((i%8 == 7) && (i<(nCnt-1))){
			putchar(LF);
			putchar(' ');
		}
	}
	putchar(LF);
}	


/********    mon_SPB    ********/

void mon_SPB( char *nParaStr )
{
	unsigned char *nAddr;
	char *nDataStr;
	unsigned char nData;
	unsigned int nCnt;
	unsigned int nOpt;
	int i;
	unsigned short nDataStart;

	nAddr = (unsigned char *)strtoul( nParaStr, &nDataStr, 16 );
	nDataStr += 1;
	nData = (unsigned char)strtoul( nDataStr, &nDataStr, 16 );
	nDataStr += 1;
	nCnt = (unsigned int)strtoul( nDataStr, &nDataStr, 10 );
	nDataStr += 1;
	nOpt = (unsigned int)strtoul( nDataStr, NULL, 10 );


	//f[^Zbg
	nDataStart = nData;
		
	for( i=0; i<nCnt; i++ ){
		*nAddr = nData;
		nData += nOpt;
	}

	putchar(LF);
	putchar(' ');
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(": ");
	hextochar_disp(2, (unsigned long)nDataStart);
	putchar(' ');
	hextochar_disp(2, (unsigned long)(nData - nOpt));
	putchar(' ');
	dectochar_disp(5, (unsigned long)nCnt);
	putchar(' ');
	dectochar_disp(3, (unsigned long)nOpt);
	putchar(LF);
}


/********    mon_SPW    ********/

void mon_SPW( char *nParaStr )
{
	unsigned short *nAddr;
	char *nDataStr;
	unsigned short nData;
	unsigned int nCnt;
	unsigned int nOpt;
	int i;
	unsigned short nDataStart;

	nAddr = (unsigned short *)strtoul( nParaStr, &nDataStr, 16 );
	nDataStr += 1;
	nData = (unsigned short)strtoul( nDataStr, &nDataStr, 16 );
	nDataStr += 1;
	nCnt = (unsigned int)strtoul( nDataStr, &nDataStr, 10 );
	nDataStr += 1;
	nOpt = (unsigned int)strtoul( nDataStr, NULL, 10 );


	//f[^Zbg
	nDataStart = nData;
		
	for( i=0; i<nCnt; i++ ){
		*nAddr = nData;
		nData += nOpt;
	}

	putchar(LF);
	putchar(' ');
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(": ");
	hextochar_disp(4, (unsigned long)nDataStart);
	putchar(' ');
	hextochar_disp(4, (unsigned long)(nData - nOpt));
	putchar(' ');
	dectochar_disp(5, (unsigned long)nCnt);
	putchar(' ');
	dectochar_disp(5, (unsigned long)nOpt);
	putchar(LF);
}


/********    mon_D2B    ********/

void mon_D2B( char *nParaStr )
{
	unsigned char *nAddr;
	char *nCntStr;

	nAddr = (unsigned char *)strtoul( nParaStr, &nCntStr, 16 );

	putchar(LF);
	putchar(' ');
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(" : ");
	
	hextobin_disp(2, (unsigned long)(*nAddr));
	putchar(LF);
}	


/********    mon_D2W    ********/

void mon_D2W( char *nParaStr )
{
	unsigned short *nAddr;
	char *nCntStr;

	nAddr = (unsigned short *)strtoul( nParaStr, &nCntStr, 16 );

	putchar(LF);
	putchar(' ');
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(" : ");
	
	hextobin_disp(4, (unsigned long)(*nAddr));
	putchar(LF);
}	


/********    mon_ANDB    ********/

void mon_ANDB(char *nParaStr){

	unsigned char	*nAddr;
	char			*nDataStr;
	unsigned char	nData;
	unsigned char	rData;

	nAddr = (unsigned char *)strtoul(nParaStr, &nDataStr, 16);
	nDataStr += 1;
	
	nData = (unsigned char)strtoul(nDataStr, NULL, 16);

	rData = *nAddr;
	*nAddr = nData & rData;

	putchar(LF);
	sputs(" Data set 0x");
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(" <-- &= 0x");
	hextochar_disp(2, (unsigned long)(nData));
	putchar(LF);
}


/********    mon_ANDW    ********/

void mon_ANDW(char *nParaStr){

	unsigned short	*nAddr;
	char			*nDataStr;
	unsigned short	nData;
	unsigned short	rData;

	nAddr = (unsigned short *)strtoul(nParaStr, &nDataStr, 16);
	nDataStr += 1;
	
	nData = (unsigned short)strtoul(nDataStr, NULL, 16);

	rData = *nAddr;
	*nAddr = nData & rData;

	putchar(LF);
	sputs(" Data set 0x");
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(" <-- &= 0x");
	hextochar_disp(4, (unsigned long)(nData));
	putchar(LF);
}


/********    mon_ORB    ********/

void mon_ORB(char *nParaStr){

	unsigned char	*nAddr;
	char			*nDataStr;
	unsigned char	nData;
	unsigned char	rData;

	nAddr = (unsigned char *)strtoul(nParaStr, &nDataStr, 16);
	nDataStr += 1;
	
	nData = (unsigned char)strtoul(nDataStr, NULL, 16);

	rData = *nAddr;
	*nAddr = nData | rData;

	putchar(LF);
	sputs(" Data set 0x");
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(" <-- |= 0x");
	hextochar_disp(2, (unsigned long)(nData));
	putchar(LF);
}


/********    mon_ORW    ********/

void mon_ORW(char *nParaStr){

	unsigned short	*nAddr;
	char			*nDataStr;
	unsigned short	nData;
	unsigned short	rData;

	nAddr = (unsigned short *)strtoul(nParaStr, &nDataStr, 16);
	nDataStr += 1;
	
	nData = (unsigned short)strtoul(nDataStr, NULL, 16);

	rData = *nAddr;
	*nAddr = nData | rData;

	putchar(LF);
	sputs(" Data set 0x");
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(" <-- |= 0x");
	hextochar_disp(4, (unsigned long)(nData));
	putchar(LF);
}


/********    mon_CHKB    ********/

void mon_CHKB(char *nParaStr){

	unsigned char	 *nAddr;
	char			 *nDataStr;
	unsigned char	 Mask;
	unsigned char	 Pattern;

	nAddr = (unsigned char *)strtoul( nParaStr, &nDataStr, 16 );
	nDataStr += 1;
	Mask = (unsigned char)strtoul( nDataStr, &nDataStr, 16 );
	nDataStr += 1;
	Pattern = (unsigned char)strtoul( nDataStr, NULL, 16 );

	while(Pattern != (*nAddr & Mask)){
		;
	}
}
	
	
/********    mon_CHKW    ********/

void mon_CHKW(char *nParaStr){

	unsigned short	 *nAddr;
	char			 *nDataStr;
	unsigned short	 Mask;
	unsigned short	 Pattern;

	nAddr = (unsigned short *)strtoul( nParaStr, &nDataStr, 16 );
	nDataStr += 1;
	Mask = (unsigned short)strtoul( nDataStr, &nDataStr, 16 );
	nDataStr += 1;
	Pattern = (unsigned short)strtoul( nDataStr, NULL, 16 );

	while(Pattern != (*nAddr & Mask)){
		;
	}
}


/********    mon_MASKB    ********/

void mon_MASKB(char *nParaStr){

	unsigned char	 *nAddr;
	char			 *nDataStr;
	unsigned char	 Mask;
	unsigned char	 Data;

	nAddr = (unsigned char *)strtoul( nParaStr, &nDataStr, 16 );
	nDataStr += 1;
	Mask = (unsigned char)strtoul( nDataStr, NULL, 16 );

	Data = *nAddr & Mask;

	putchar(LF);
	putchar(' ');
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(" : ");
	
	hextobin_disp(2, (unsigned long)Data);
	putchar(LF);
}
	
	
/********    mon_MASKW    ********/

void mon_MASKW(char *nParaStr){

	unsigned short	 *nAddr;
	char			 *nDataStr;
	unsigned short	 Mask;
	unsigned short	 Data;

	nAddr = (unsigned short *)strtoul( nParaStr, &nDataStr, 16 );
	nDataStr += 1;
	Mask = (unsigned short)strtoul( nDataStr, NULL, 16 );

	Data = *nAddr & Mask;

	putchar(LF);
	putchar(' ');
	hextochar_disp(6, (unsigned long)nAddr);
	sputs(" : ");
	
	hextobin_disp(4, (unsigned long)Data);
	putchar(LF);
}


/********    mon_CSET   ********/

void mon_CSET(char *nParaStr){

	unsigned char	Color;
	
	Color = (unsigned char)strtoul(nParaStr, NULL, 10);
	
	if(Color < 8){
		sputs("\033[3");
		putchar(Color + 0x30);
		sputs("m");
	}
}


/********    mon_help  ********/

void mon_help(void){

	putchar(LF);
	puts(" DB  : dump BYTE  [DB <add> <number>]");
	puts(" DW  : dump WORD  [DW <add> <number>]");
	puts(" SB  : set  BYTE  [SB <add> <data>]");
	puts(" SW  : set  WORD  [SW <add> <data>]");

	putchar(LF);
	puts(" DPB : dump PORT BYTE  [DPB <add> <number>]");
	puts(" DPW : dump PORT WORD  [DPW <add> <number>]");
	puts(" SPB : set  PORT BYTE  [SPB <add> <data> <number> <inc data>]");
	puts(" SPW : set  PORT WORD  [SPW <add> <data> <number> <inc data>]");
	putchar(LF);
	puts(" D2B : dump PORT (Binary) BYTE [D2B <add>]");
	puts(" D2W : dump PORT (Binary) WORD [D2W <add>]");

	putchar(LF);
	puts(" ANDB: and BYTE  [ANDB <add> <data>]");
	puts(" ANDW: and WORD  [ANDW <add> <data>]");
	puts(" ORB : or  BYTE  [ORB <add> <data>]");
	puts(" ORW : or  WORD  [ORW <add> <data>]");
	putchar(LF);
	puts(" MASKB: mask BYTE  [MASKB <add> <mask>]");
	puts(" MASKW: mask BYTE  [MASKW <add> <mask>]");
	puts(" CHKB: check BYTE  [CHKB <add> <mask> <pattern>]");
	puts(" CHKW: check BYTE  [CHKW <add> <mask> <pattern>]");


	putchar(LF);
	puts(" CSET: set disp color  [CSET <number>] 0:black, 1:red,   2:green,      3:yellow");
	puts("                                       4:blue,  5:violet 6:light blue, 7:white");
	putchar(LF);
	puts(" //  : echo only");
	puts(" ?   : command help (this list)");

}


